# -*- coding: utf-8 -*-

routes = [
    {
        "name": "Home",
        "url": "Home"
    },
    {
        "name": "Counter",
        "url": "Counter"
    },
    {
        "name": "Hello",
        "url": "Hello"
    }
]
